﻿<#
    Company: Onevinn AB
    Name: PrepareDisk.ps1
    Author: Johan Schrewelius
    Updated: Sassan Fanai
    Date: 2019-10-01
    Version: 1.2
    Usage: Run as prestart command in ConfigMgr Boot image to create and format partitions, if disk is "raw".
#>
function WriteLog {
    param(
    [Parameter(Mandatory)]
    [string]$LogText,
    [Parameter(Mandatory=$true)]
    $Component,
    [Parameter(Mandatory)]
    [ValidateNotNullOrEmpty()]
    [ValidateSet('Info','Warning','Error','Verbose')]
    [string]$Type,
    [Parameter(Mandatory)]
    [string]$LogFileName,
    [Parameter(Mandatory)]
    [string]$FileName
    )

    switch ($Type)
    {
        "Info"      { $typeint = 1 }
        "Warning"   { $typeint = 2 }
        "Error"     { $typeint = 3 }
        "Verbose"   { $typeint = 4 }
    }

    $time = Get-Date -f "HH:mm:ss.ffffff"
    $date = Get-Date -f "MM-dd-yyyy"
    $ParsedLog = "<![LOG[$($LogText)]LOG]!><time=`"$($time)`" date=`"$($date)`" component=`"$($Component)`" context=`"`" type=`"$($typeint)`" thread=`"$($pid)`" file=`"$($FileName)`">"
    $ParsedLog | Out-File -FilePath "$LogFileName" -Append -Encoding utf8
}

$tsenv = New-Object -COMObject Microsoft.SMS.TSEnvironment
$IsUefi = $tsenv.Value("_SMSTSBootUEFI").ToLower().Equals("true")
$Self = $($MyInvocation.MyCommand.Name)
$LogPath = "$env:TEMP\SMSTSLog"
$LogFile = "$LogPath\Onevinn." + $Self.Replace(".ps1", ".log")
WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Script and Com initiated, log started" -Type Info


$Partitions = (gwmi -Namespace 'ROOT\Cimv2' -Query "SELECT * FROM Win32_DiskPartition").Type
#$Partitions = (gwmi -Namespace 'ROOT\Cimv2' -Query "SELECT * FROM Win32_DiskDrive WHERE DeviceID Like '%PHYSICALDRIVE0'").Partitions
$Vols = gwmi -Namespace 'ROOT\Cimv2' -Query "SELECT * FROM Win32_LogicalDisk WHERE DeviceID != 'X:' AND DriveType = '3'"

#WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Number of parititions: $Partitions" -Type Info
WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Volumes: $($Vols.DeviceID)" -Type Info

if ($Vols) {
    If (Get-ChildItem $Vols.DeviceID) {
        WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Found at least one readable volume" -Type Info
    }
    else {
        WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Found at least one volume but could not read it, will continue to format and partition drive" -Type Warning
        $NoExit = $true
    }

    if (!$IsUefi -and $Partitions -like "GPT*") {
        WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Disks are GPT but system is MBR?" -Type Warning
        $NoExit = $true
    }
    if (!$NoExit) {
        WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Exiting script" -Type Info
        exit 0
    }
}


else {
    WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "No readable volume found, will continue to format and partition drive" -Type Info
}


if (!$IsUefi) {

$DPCommands =
@"
select disk 0
clean
create partition primary size=350
assign letter=S
format fs=ntfs quick label="System Reserved"
active
create partition primary
format fs=ntfs quick label="Windows"
Assign Letter=C
shrink desired=984 minimum=984
create partition primary
format quick fs=ntfs label="Recovery"
set id=27
"@

WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Creating legacy partitions" -Type Info
WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "$DPCommands" -Type Info
}

if ($IsUefi) {

$DPCommands =
@"
select disk 0
clean
convert gpt
create partition efi size=260
format quick fs=fat32 label="System"
create partition msr size=128
create partition primary
format quick fs=ntfs label="Windows"
shrink desired=984 minimum=984
assign letter=C
create partition primary
format quick fs=ntfs label="Recovery"
set id="de94bba4-06d1-4d40-a16a-bfd50179d6ac"
gpt attributes=0x8000000000000001
"@
WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "Creating UEFI partitions" -Type Info
WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "$DPCommands" -Type Info

}

$DPCommands | Out-File -FilePath "diskpart.txt" -Encoding ascii -Force
Start-Process -FilePath "diskpart.exe" -ArgumentList @("/S diskpart.txt") -WindowStyle Hidden -Wait -EA SilentlyContinue
WriteLog -LogFileName "$LogFile" -Component "RunPowerShellScript" -FileName "$Self" -LogText "DONE" -Type Info